% ---------------------------------------------------------------------------
%
% analysis_mobs
%
% program to examine effects of analysing multiple idealised observations,
% given a forecast background field and assumed error covariance functions
% this version features (up to) two different types of observations
%
% P.J. Smith July 2014
% Apadpted from IDL/PWAVE procedure written by R. Swinbank 2002
% -------------------------------------------------------------------------
  

% -----------------------------------------------------------------------------
% MAIN program
% -----------------------------------------------------------------------------

clear all

msgtext = {'**** ANALYSIS_MOBS - Toy Analysis System ****',...
           '  ', ...
           'Constructs 1-D analysis from multiple idealised observations with specified values and error characteristics',...
           '  ',...
           'In this programme, we use two different ob types (A and B).', ...
           'The number of obs of each type should be in range 0 to 7.',...
           '  ',...
           'You may change the analysis parameters at the prompt ...', ...
           '  ',...
           'To change ARRAYS you need to type in the correct number of new values',...
           '  '};
           
name = 'analysis_mobs';       

uiwait(msgbox(msgtext,name));


% set x values at which equations are solved
x0=0.;
xn=10.;
nx=100; % will have nx+1 points
dx=(xn-x0)/nx;
x=[x0:dx:xn]'; % x axis points

% initialise analysis vectors
xdim = length(x);
a  = zeros(size(x));    % analysis for all obs case
at = zeros(xdim,2);  % analysis for separate obs cases

fb = zeros(size(x)); % background

% set default values
noa = 3 ; % no. of obs in set A
nob = 3 ; % no. of obs in set B
% max no. of obs in each set; 
noax = 7;
nobx = 7;
% set initial values of ob locations
xoa = [4.0, 5.0, 6.0]';
xob = [4.0, 5.0, 6.0]';

oba = 1.0*ones(noa,1);
obb = -1.0*ones(nob,1);

% error standard deviation and correlation lengths
sigoa = 2.0;
Loa   = 0.0;
sigob = 1.0;
Lob   = 2.0;

% background error standard deviaton and correlation length
sigf = 1.0;
Lf   = 1.0;

% REQUEST USER INPUT:
l_default = menu_asl('Use default parameter values?','Yes','No');

if (l_default==2)

% OBSERVATION VALUES & STATISTICS

% Type A obs 
name = 'TYPE-A OBSERVATIONS'; 
Aprompt = {'Enter number of type A obs, (max 7, enter 0 if none): ',...
          'Enter locations of observations (between 0 and 10)',...
          'Enter values of observations (between -2 and 2)',...
          'Enter observation error for type A, sigoa: ',...
          'Enter observation error correlation length Loa (0.0 for uncorrelated errors):'};
df = {num2str(noa),num2str(xoa'),num2str(oba'),num2str(sigoa),num2str(Loa)};

inputAok = 0;

while ~inputAok
    
answerA = inputdlg(Aprompt,name,1,df); 

if isempty(answerA)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
else
 noa = str2num(answerA{1});
 xoa = str2num(answerA{2})';
 oba = str2num(answerA{3})';
 
 if (~isempty(find(xoa>10)) | ~isempty(find(xoa<0)) | ~isempty(find(abs(oba)>2)) )
  errorflag = 1;
 else
  errorflag = 0;
 end    
 
 sigoa = str2num(answerA{4});
 Loa   = str2num(answerA{5});
end

if errorflag
 uiwait(msgbox('invalid observation locations/ values entered, please try again'));   
elseif (noa~=0 & (noa>noax | length(xoa)~=noa | length(oba)~=noa)) 
 uiwait(msgbox('incorrect number of observation locations/ values entered, please try again')); 
else
 inputAok = 1;   
end    
 
 
end % while


% Type B obs 
name = 'TYPE-B OBSERVATIONS'; 
Bprompt = {'Enter number of type B obs, (max 7, enter 0 if none): ',...
          'Enter locations of observations (bewteen 0 and 10)',...
          'Enter values of observations (between -2 and 2)',...
          'Enter observation error for type B, sigob: ',...
          'Enter observation error correlation length Lob (0.0 for uncorrelated errors):'};
df = {num2str(nob),num2str(xob'),num2str(obb'),num2str(sigob),num2str(Lob)};

inputBok = 0;

while ~inputBok
    
answerB = inputdlg(Bprompt,name,1,df); 

if isempty(answerB)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return 
else
 nob = str2num(answerB{1});
 xob = str2num(answerB{2})';
 obb = str2num(answerB{3})';
 
 if (~isempty(find(xob>10)) | ~isempty(find(xob<0)) | ~isempty(find(abs(obb)>2)) )
  errorflag = 1;
 else
  errorflag = 0;
 end    
 
 sigob = str2num(answerB{4});
 Lob   = str2num(answerB{5});
end

if errorflag
 uiwait(msgbox('invalid observation locations/ values entered, please try again')); 
elseif (nob~=0 & (nob>nobx | length(xob)~=nob | length(obb)~=nob))
 uiwait(msgbox('incorrect number of observation locations/ values entered, please try again')); 
else
 inputBok = 1;   
end    
 
 
end % while

% BACKGROUND
 bgprompt={'Enter value of background error sigf:',...
           'Enter value of background error correlation length, Lf (put 0.0 for uncorrelated errors):'};
 name='BACKGROUND';
 defaultanswer={num2str(sigf),num2str(Lf)};
 bganswer=inputdlg(bgprompt,name,1,defaultanswer); %,options); 
 
 if isempty(bganswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
 else
  sigf = str2num(bganswer{1});
  Lf   = str2num(bganswer{2});     
 end

end % if l_default==2


nt = 1; % no. of ob types (reset later if noa & nob are both non-zero)
no = noa+nob; % total number of observations
xo = zeros(no,1); % x positions of combined obs
fo = zeros(no,1); % Hfb
ob = zeros(no,1); % vector of combined observations 
od = zeros(no,1); % y-Hfb 

% concatenate ob positions and values
if (noa>0)
  xo(1:noa)=xoa;
  ob(1:noa)=oba;
end   

if (nob>0)
  xo(noa+1:no)=xob;
  ob(noa+1:no)=obb;
end   

% forecast fields at obs locations & y-Hfb differences
fo = interp1(x,fb,xo,'linear'); % Hfb -> this will always be zero since fb=0
od = ob - fo;  % y-Hfb

% Set up M = (HBH^T + R), for all obs
% plus equivalents for each ob type on its own
M = zeros(no);

% background error correlations in obs space, HBH^T
rho = soar2(xo,xo,Lf);  
M   = sigf*sigf*rho;
    
% observation correlations R (assume no correlations between obs of different types)

% TYPE A
if noa > 0
  Ma   = zeros(noa);
  rhoa = soar2(xo(1:noa),xo(1:noa),Loa);
  
  M(1:noa,1:noa) = M(1:noa,1:noa) + sigoa*sigoa*rhoa;  % (HBH^T + R)
  Ma = M(1:noa,1:noa);  
  
  sigo = sigoa;  % parameters used if only obs of type A
  Lo   = Loa;
end

 % TYPE B
if nob>0
  Mb   = zeros(nob);
  rhob = soar2(xo(noa+1:no),xo(noa+1:no),Lob);    
  
  M(noa+1:no,noa+1:no) = M(noa+1:no,noa+1:no) + sigob*sigob*rhob; % (HBH^T + R)
  Mb = M(noa+1:no,noa+1:no);
  
  sigo = sigob;  % parameters used if only obs of type B
  Lo   = Lob;  
end  

% get inverse of M
Mi=inv(M);

% calculate separate inverses for each ob type 
if (noa > 0 & nob > 0)
  nt  = 2;
  Mai = inv(Ma);
  Mbi = inv(Mb); 
end

if (nt==1)
 % analysis for individual obs locations (needed if only one obs type)
 a1 = zeros(xdim,no);
 % For "single ob" analysis wt for 1 ob 1/(var_b+var_o)
 W1 = 1.0/(sigf*sigf+sigo*sigo); 
end

% evaluate analysis
  
rho = soar2(x,xo,Lf);   
cf  = sigf*sigf*rho;   % BH^T  (xdim x no)
   
z = Mi*od;   % (HBH^T + R)^-1*(y-Hfb)
    
% analysis (with all obs)
a = fb + cf*z; % xa = fb + BH^T(HBH^T + R)^-1*(y-Hfb)
    
if nt==1 % single obs type
 for i=1:no   
  a1(:,i) = fb + cf(:,i)*W1*od(i); 
 end 
end
      
if nt==2  % compute separate analyses for obs types A & B
    
 rhoa = soar2(x,xo(1:noa),Lf);   
 cfa  = sigf*sigf*rhoa;
 z    = Mai*od(1:noa);
 
 at(:,1) = fb + cfa*z;
    
 rhob = soar2(x,xo(noa+1:no),Lf);   
 cfb  = sigf*sigf*rhob;
 z    = Mbi*od(noa+1:no);
 
 at(:,2) = fb + cfb*z;
       
end % if nt==2


% plot results
h = figure; hold on
title('analysis\_mobs')
if (nt==1)
 xtext = ['sigo = ' num2str(sigo) ', Lo = ' num2str(Lo) ', sigf = ' num2str(sigf) ', Lf = ' num2str(Lf)];
 ltext = {'observations','analysis: all obs','analysis: single obs'};
else
 xtext = ['sigoa = ' num2str(sigoa) ', Loa = ' num2str(Loa) ', sigob = ' num2str(sigob) ', Lob = ' num2str(Lob) ', sigf = ' num2str(sigf) ', Lf = ' num2str(Lf)];   
 ltext = {'observations: type A','observations: type B','analysis: all obs','analysis: obs type A', 'analysis: obs type B'};   
end   

xlabel(xtext)

if (noa>0)
 plot(xo(1:noa),ob(1:noa),'kx','linewidth',1,'MarkerSize',8)  % observations
end

if (nob>0)
 plot(xo(noa+1:no),ob(noa+1:no),'k+','linewidth',1,'MarkerSize',8)  % observations
end

plot(x,a,'r--','linewidth',2)

if (nt==1)
 for i=1:no
   plot(x,a1(:,i),'b:','linewidth',2);  
   if (i==1); legend(ltext); end
 end
else  
 plot(x,at(:,1),'b:','linewidth',2)
 plot(x,at(:,2),'g:','linewidth',2)
 legend(ltext)
end


plot(x,fb,'k','linewidth',1)

set(gca,'xtick',[0:2:10],'xlim',[0 10],'ylim',[-2 2], 'ytick',[-2:1:2],'XMinorTick','on','YMinorTick','on');
box on 

l_save = menu_asl('Do you wish to save this plot?','Yes','No');

if (l_save==1)
 prompt = {'please enter a filename:'};
 name   = 'save figure';
 df     = {'analysis_mobs_plot1'} ;
 pname  = inputdlg(prompt,name,1,df);
 
 if isempty(pname)
  disp('*** user options cancelled ****')
  disp('please try again ...') 
  break
 end    
 
 format = 'fig';
 saveas(h,char(pname),format);   
    
end

% Write out options chosen
%
disp('  ')
disp('********************************************************************')
disp('  ')
disp('SUMMARY OF CHOSEN PARAMETERS:')
disp('  ')
disp(['background error standard deviation, sigf = ' num2str(sigf)])
disp(['background error correlation lengthscale, Lf   = ' num2str(Lf)])
disp('  ')
disp('Observation type A:')
disp('-------------------')
disp('  ')
disp(['number of observations: ' num2str(noa)])
disp(['observation locations : ' num2str(xoa')])
disp(['observation values    : ' num2str(oba')])
disp(['observation error standard deviation, sigoa = ' num2str(sigoa)])
disp(['observation error correlation lengthscale, Loa   = ' num2str(Loa)])
disp('  ')
disp('Observation type B:')
disp('-------------------')
disp('  ')
disp(['number of observations: ' num2str(nob)])
disp(['observation locations : ' num2str(xob')])
disp(['observation values    : ' num2str(obb')])
disp(['observation error standard deviation, sigob = ' num2str(sigob)])
disp(['observation error correlation lengthscale, Lob   = ' num2str(Lob)])
disp('  ')
disp('********************************************************************')
%%%%%%%%%%  END OF PROGRAM  %%%%%%%%%%

