function rndx = frand(x,sig,L,norder,seed,test,shift)

% Generate a randomly varying series, at regularly spaced points x.
% The series has a nominal mean of 0.0, standard deviation of sig, and a
% correlation length L.
% The series is generated using a recursive filter technique applied to a
% random number series, which originally has a sigma of 1.0
% if shift is set, then series is shifted (to the left) by shift units
%
% P.J. Smith July 2014
% Apadpted from IDL/PWAVE procedure written by R. Swinbank 2002
% -------------------------------------------------------------------------
 
nx = length(x);
dx = abs(x(2)-x(1));

% work with extra long arrays, but only return last nx values generated
% add an extra nst points
nst = 500;

if (nargin > 6) & (shift~=0) 
 if test
   disp('applying shift')
 end
 nd  = shift/dx;
 nst = nst + nd;
end

nr = nx + nst;

% generate unfiltered sequence of normally distributed random numbers
randn('seed',seed);  % reset seed
r = zeros(nr,1);
r = randn(size(r));   
r(1) = 0.0; % force series to start at zero! 

% get standard deviation of unfiltered series (nominally 1.0, with mean of
% 0.0)
rm  = mean(r);
rsd = std(r,1); % 2nd moment

if test 
  display(['Unfiltered mean & std: ' num2str(rm) ', ' num2str(rsd)])
end

if (norder ~= 1 & norder ~= 2)
  error(['norder is not 1 or 2; it is ',num2str(norder)])
end  

if (L > 2.0*dx) & (L > 0.0)
 % recursive filter
 % filter weight is derived from L and dx, assuming dx/L<<1.0
  w = 1.0 - dx/L;
  
  if test
   disp(['Recursive filter weight, order: ' num2str(w) ', ' num2str(norder)])
  end
  
  for j=1:norder
    for i=2:nr
      r(i) = w*r(i-1)+(1.0-w)*r(i);
    end
  end
  
  % copy last nx points for output
  f = zeros(size(x));
  f = r(nr-nx:nr-1);  

  % get SD of filtered series, and rescale to requested value    
  fm  = mean(f);
  fsd = std(f,1);
  
  if test 
   disp(['Filtered mean & std: ' num2str(fm) ', ' num2str(fsd)])
  end
  
  % Adjust series to give requested SD (& zero mean)
  % try calculating scaling factor
  if (norder==1)
    wp = w;
  elseif (norder==2)
    wp = sqrt(w);
  end
  
  wfact = sig*sqrt((1.0+wp)/(1.0-wp));  % theoretical scaling
  
  if test 
   disp(['Scaling calculated: ' num2str(wfact)])
  end
  
  f = f - fm;  % subtract mean
  
elseif (L > 0.0) % &  (L <= 2.0*dx)
  error('Correlation length is too short!')
  
else % L <= 0.0 - no filtering
  wfact = sig;
  f = zeros(size(x));
  f = r(nr-nx:nr-1);
end

rndx = f*wfact;

  if test 
   disp(['series mean & std: ' num2str(mean(rndx)) ', ' num2str(std(rndx,1))])
  end


end % function